@@ -1,5 +1,6 @@
 mod cli;
 mod config;
+mod config_watcher;
 mod constants;
 mod helper;
 mod multi_map;
@@ -9,11 +10,12 @@ mod transport;
 pub use cli::Cli;
 use cli::KeypairType;
 pub use config::Config;
+use config_watcher::ServiceChange;
 pub use constants::UDP_BUFFER_SIZE;
 
-use anyhow::{anyhow, Result};
-use tokio::sync::broadcast;
-use tracing::debug;
+use anyhow::Result;
+use tokio::sync::{broadcast, mpsc};
+use tracing::{debug, info};
 
 #[cfg(feature = "client")]
 mod client;
@@ -25,6 +27,8 @@ mod server;
 #[cfg(feature = "server")]
 use server::run_server;
 
+use crate::config_watcher::{ConfigChange, ConfigWatcherHandle};
+
 const DEFAULT_CURVE: KeypairType = KeypairType::X25519;
 
 fn get_str_from_keypair_type(curve: KeypairType) -> &'static str {
@@ -56,33 +60,83 @@ fn genkey(curve: Option<KeypairType>) -> Result<()> {
     crate::helper::feature_not_compile("nosie")
 }
 
-pub async fn run(args: &Cli, shutdown_rx: broadcast::Receiver<bool>) -> Result<()> {
+pub async fn run(args: Cli, shutdown_rx: broadcast::Receiver<bool>) -> Result<()> {
     if args.genkey.is_some() {
         return genkey(args.genkey.unwrap());
     }
 
-    let config = Config::from_file(args.config_path.as_ref().unwrap()).await?;
-
-    debug!("{:?}", config);
-
     // Raise `nofile` limit on linux and mac
     fdlimit::raise_fd_limit();
 
-    match determine_run_mode(&config, args) {
-        RunMode::Undetermine => Err(anyhow!("Cannot determine running as a server or a client")),
+    // Spawn a config watcher. The watcher will send a initial signal to start the instance with a config
+    let config_path = args.config_path.as_ref().unwrap();
+    let mut cfg_watcher = ConfigWatcherHandle::new(config_path, shutdown_rx).await?;
+
+    // shutdown_tx owns the instance
+    let (shutdown_tx, _) = broadcast::channel(1);
+
+    // (The join handle of the last instance, The service update channel sender)
+    let mut last_instance: Option<(tokio::task::JoinHandle<_>, mpsc::Sender<ServiceChange>)> = None;
+
+    while let Some(e) = cfg_watcher.event_rx.recv().await {
+        match e {
+            ConfigChange::General(config) => {
+                if let Some((i, _)) = last_instance {
+                    info!("General configuration change detected. Restarting...");
+                    shutdown_tx.send(true)?;
+                    i.await?;
+                }
+
+                debug!("{:?}", config);
+
+                let (service_update_tx, service_update_rx) = mpsc::channel(1024);
+
+                last_instance = Some((
+                    tokio::spawn(run_instance(
+                        *(config.clone()),
+                        args.clone(),
+                        shutdown_tx.subscribe(),
+                        service_update_rx,
+                    )),
+                    service_update_tx,
+                ));
+            }
+            ConfigChange::ServiceChange(service_event) => {
+                info!("Service change detcted. {:?}", service_event);
+                if let Some((_, service_update_tx)) = &last_instance {
+                    let _ = service_update_tx.send(service_event).await;
+                }
+            }
+        }
+    }
+
+    let _ = shutdown_tx.send(true);
+
+    Ok(())
+}
+
+async fn run_instance(
+    config: Config,
+    args: Cli,
+    shutdown_rx: broadcast::Receiver<bool>,
+    service_update: mpsc::Receiver<ServiceChange>,
+) {
+    let ret: Result<()> = match determine_run_mode(&config, &args) {
+        RunMode::Undetermine => panic!("Cannot determine running as a server or a client"),
         RunMode::Client => {
             #[cfg(not(feature = "client"))]
             crate::helper::feature_not_compile("client");
             #[cfg(feature = "client")]
-            run_client(&config, shutdown_rx).await
+            run_client(&config, shutdown_rx, service_update).await
         }
         RunMode::Server => {
             #[cfg(not(feature = "server"))]
             crate::helper::feature_not_compile("server");
             #[cfg(feature = "server")]
-            run_server(&config, shutdown_rx).await
+            run_server(&config, shutdown_rx, service_update).await
         }
-    }
+    };
+    ret.unwrap();
 }
 
 #[derive(PartialEq, Eq, Debug)]
