@@ -1,4 +1,5 @@
 use crate::config::{Config, ServerConfig, ServerServiceConfig, ServiceType, TransportType};
+use crate::config_watcher::ServiceChange;
 use crate::constants::{listen_backoff, UDP_BUFFER_SIZE};
 use crate::multi_map::MultiMap;
 use crate::protocol::Hello::{ControlChannelHello, DataChannelHello};
@@ -30,12 +31,16 @@ use crate::transport::TlsTransport;
 type ServiceDigest = protocol::Digest; // SHA256 of a service name
 type Nonce = protocol::Digest; // Also called `session_key`
 
-const TCP_POOL_SIZE: usize = 64; // The number of cached connections for TCP servies
+const TCP_POOL_SIZE: usize = 8; // The number of cached connections for TCP servies
 const UDP_POOL_SIZE: usize = 2; // The number of cached connections for UDP services
 const CHAN_SIZE: usize = 2048; // The capacity of various chans
 
 // The entrypoint of running a server
-pub async fn run_server(config: &Config, shutdown_rx: broadcast::Receiver<bool>) -> Result<()> {
+pub async fn run_server(
+    config: &Config,
+    shutdown_rx: broadcast::Receiver<bool>,
+    service_rx: mpsc::Receiver<ServiceChange>,
+) -> Result<()> {
     let config = match &config.server {
             Some(config) => config,
             None => {
@@ -43,17 +48,16 @@ pub async fn run_server(config: &Config, shutdown_rx: broadcast::Receiver<bool>)
             }
         };
 
-    //TODO: Maybe use a Box<dyn trait> here to reduce duplicated code
     match config.transport.transport_type {
         TransportType::Tcp => {
             let mut server = Server::<TcpTransport>::from(config).await?;
-            server.run(shutdown_rx).await?;
+            server.run(shutdown_rx, service_rx).await?;
         }
         TransportType::Tls => {
             #[cfg(feature = "tls")]
             {
                 let mut server = Server::<TlsTransport>::from(config).await?;
-                server.run(shutdown_rx).await?;
+                server.run(shutdown_rx, service_rx).await?;
             }
             #[cfg(not(feature = "tls"))]
             crate::helper::feature_not_compile("tls")
@@ -62,7 +66,7 @@ pub async fn run_server(config: &Config, shutdown_rx: broadcast::Receiver<bool>)
             #[cfg(feature = "noise")]
             {
                 let mut server = Server::<NoiseTransport>::from(config).await?;
-                server.run(shutdown_rx).await?;
+                server.run(shutdown_rx, service_rx).await?;
             }
             #[cfg(not(feature = "noise"))]
             crate::helper::feature_not_compile("noise")
@@ -81,8 +85,6 @@ struct Server<'a, T: Transport> {
     // `[server]` config
     config: &'a ServerConfig,
 
-    // TODO: Maybe the rwlock is unnecessary.
-    // Keep it until the hot reloading feature is implemented
     // `[server.services]` config, indexed by ServiceDigest
     services: Arc<RwLock<HashMap<ServiceDigest, ServerServiceConfig>>>,
     // Collection of contorl channels
@@ -114,7 +116,11 @@ impl<'a, T: 'static + Transport> Server<'a, T> {
     }
 
     // The entry point of Server
-    pub async fn run(&mut self, mut shutdown_rx: broadcast::Receiver<bool>) -> Result<()> {
+    pub async fn run(
+        &mut self,
+        mut shutdown_rx: broadcast::Receiver<bool>,
+        mut service_rx: mpsc::Receiver<ServiceChange>,
+    ) -> Result<()> {
         // Listen at `server.bind_addr`
         let l = self
             .transport
@@ -172,12 +178,38 @@ impl<'a, T: 'static + Transport> Server<'a, T> {
                 _ = shutdown_rx.recv() => {
                     info!("Shuting down gracefully...");
                     break;
+                },
+                e = service_rx.recv() => {
+                    if let Some(e) = e {
+                        self.handle_hot_reload(e).await;
+                    }
                 }
             }
         }
 
         Ok(())
     }
+
+    async fn handle_hot_reload(&mut self, e: ServiceChange) {
+        match e {
+            ServiceChange::ServerAdd(s) => {
+                let hash = protocol::digest(s.name.as_bytes());
+                let mut wg = self.services.write().await;
+                let _ = wg.insert(hash, s);
+
+                let mut wg = self.control_channels.write().await;
+                let _ = wg.remove1(&hash);
+            }
+            ServiceChange::ServerDelete(s) => {
+                let hash = protocol::digest(s.as_bytes());
+                let _ = self.services.write().await.remove(&hash);
+
+                let mut wg = self.control_channels.write().await;
+                let _ = wg.remove1(&hash);
+            }
+            _ => (),
+        }
+    }
 }
 
 // Handle connections to `server.bind_addr`
@@ -297,7 +329,6 @@ async fn do_data_channel_handshake<T: 'static + Transport>(
             handle.data_ch_tx.send(conn).await?;
         }
         None => {
-            // TODO: Maybe print IP here
             warn!("Data channel has incorrect nonce");
         }
     }
@@ -305,11 +336,8 @@ async fn do_data_channel_handshake<T: 'static + Transport>(
 }
 
 pub struct ControlChannelHandle<T: Transport> {
-    // Shutdown the control channel.
-    // Not used for now, but can be used for hot reloading
-    #[allow(dead_code)]
-    shutdown_tx: broadcast::Sender<bool>,
-    //data_ch_req_tx: mpsc::Sender<bool>,
+    // Shutdown the control channel by dropping it
+    _shutdown_tx: broadcast::Sender<bool>,
     data_ch_tx: mpsc::Sender<T::Stream>,
 }
 
@@ -321,10 +349,7 @@ where
     // and the connection pool task are created.
     #[instrument(skip_all, fields(service = %service.name))]
     fn new(conn: T::Stream, service: ServerServiceConfig) -> ControlChannelHandle<T> {
-        // Save the name string for logging
-        let name = service.name.clone();
-
-        // Create a shutdown channel. The sender is not used for now, but for future use
+        // Create a shutdown channel
         let (shutdown_tx, shutdown_rx) = broadcast::channel::<bool>(1);
 
         // Store data channels
@@ -375,22 +400,20 @@ where
         };
 
         // Run the control channel
-        tokio::spawn(async move {
-            if let Err(err) = ch.run().await {
-                error!(%name, "{}", err);
+        tokio::spawn(
+            async move {
+                if let Err(err) = ch.run().await {
+                    error!("{:?}", err);
+                }
             }
-        });
+            .instrument(Span::current()),
+        );
 
         ControlChannelHandle {
-            shutdown_tx,
+            _shutdown_tx: shutdown_tx,
             data_ch_tx,
         }
     }
-
-    #[allow(dead_code)]
-    fn shutdown(self) {
-        let _ = self.shutdown_tx.send(true);
-    }
 }
 
 // Control channel, using T as the transport layer. P is TcpStream or UdpTraffic
@@ -486,10 +509,10 @@ fn tcp_listen_and_send(
                         }
                         Ok((incoming, addr)) => {
                             // For every visitor, request to create a data channel
-                            if let Err(e) = data_ch_req_tx.send(true) {
+                            if let Err(e) = data_ch_req_tx.send(true).with_context(|| "Failed to send data chan create request") {
                                 // An error indicates the control channel is broken
                                 // So break the loop
-                                error!("{}", e);
+                                error!("{:?}", e);
                                 break;
                             }
 
@@ -507,7 +530,7 @@ fn tcp_listen_and_send(
                 }
             }
         }
-    });
+    }.instrument(Span::current()));
 
     rx
 }
