@@ -15,10 +15,10 @@ use crate::common::run_rathole_server;
 
 mod common;
 
-const ECHO_SERVER_ADDR: &str = "0.0.0.0:8080";
-const PINGPONG_SERVER_ADDR: &str = "0.0.0.0:8081";
-const ECHO_SERVER_ADDR_EXPOSED: &str = "0.0.0.0:2334";
-const PINGPONG_SERVER_ADDR_EXPOSED: &str = "0.0.0.0:2335";
+const ECHO_SERVER_ADDR: &str = "127.0.0.1:8080";
+const PINGPONG_SERVER_ADDR: &str = "127.0.0.1:8081";
+const ECHO_SERVER_ADDR_EXPOSED: &str = "127.0.0.1:2334";
+const PINGPONG_SERVER_ADDR_EXPOSED: &str = "127.0.0.1:2335";
 const HITTER_NUM: usize = 4;
 
 #[derive(Clone, Copy, Debug)]
@@ -55,6 +55,8 @@ async fn tcp() -> Result<()> {
     });
 
     test("tests/for_tcp/tcp_transport.toml", Type::Tcp).await?;
+    // FIXME: Self-signed certificate on Mac requires mannual interference. Disable CI for now
+    #[cfg(not(target_os = "macos"))]
     test("tests/for_tcp/tls_transport.toml", Type::Tcp).await?;
     test("tests/for_tcp/noise_transport.toml", Type::Tcp).await?;
 
@@ -80,6 +82,8 @@ async fn udp() -> Result<()> {
     });
 
     test("tests/for_udp/tcp_transport.toml", Type::Udp).await?;
+    // See above
+    #[cfg(not(target_os = "macos"))]
     test("tests/for_udp/tls_transport.toml", Type::Udp).await?;
     test("tests/for_udp/noise_transport.toml", Type::Udp).await?;
 
@@ -94,7 +98,7 @@ async fn test(config_path: &'static str, t: Type) -> Result<()> {
     // Start the client
     info!("start the client");
     let client = tokio::spawn(async move {
-        run_rathole_client(&config_path, client_shutdown_rx)
+        run_rathole_client(config_path, client_shutdown_rx)
             .await
             .unwrap();
     });
@@ -105,7 +109,7 @@ async fn test(config_path: &'static str, t: Type) -> Result<()> {
     // Start the server
     info!("start the server");
     let server = tokio::spawn(async move {
-        run_rathole_server(&config_path, server_shutdown_rx)
+        run_rathole_server(config_path, server_shutdown_rx)
             .await
             .unwrap();
     });
@@ -126,7 +130,7 @@ async fn test(config_path: &'static str, t: Type) -> Result<()> {
     info!("restart the client");
     let client_shutdown_rx = client_shutdown_tx.subscribe();
     let client = tokio::spawn(async move {
-        run_rathole_client(&config_path, client_shutdown_rx)
+        run_rathole_client(config_path, client_shutdown_rx)
             .await
             .unwrap();
     });
@@ -147,7 +151,7 @@ async fn test(config_path: &'static str, t: Type) -> Result<()> {
     info!("restart the server");
     let server_shutdown_rx = server_shutdown_tx.subscribe();
     let server = tokio::spawn(async move {
-        run_rathole_server(&config_path, server_shutdown_rx)
+        run_rathole_server(config_path, server_shutdown_rx)
             .await
             .unwrap();
     });
@@ -214,7 +218,7 @@ async fn tcp_echo_hitter(addr: &'static str) -> Result<()> {
 }
 
 async fn udp_echo_hitter(addr: &'static str) -> Result<()> {
-    let conn = UdpSocket::bind("0.0.0.0:0").await?;
+    let conn = UdpSocket::bind("127.0.0.1:0").await?;
     conn.connect(addr).await?;
 
     let mut wr = [0u8; 128];
@@ -249,7 +253,7 @@ async fn tcp_pingpong_hitter(addr: &'static str) -> Result<()> {
 }
 
 async fn udp_pingpong_hitter(addr: &'static str) -> Result<()> {
-    let conn = UdpSocket::bind("0.0.0.0:0").await?;
+    let conn = UdpSocket::bind("127.0.0.1:0").await?;
     conn.connect(&addr).await?;
 
     let wr = PING.as_bytes();
